unit Board;

interface

{/*
 *	BOARD.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 */}

{{/*
 *	DATA.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 */}

// pascal version


{/*
 *	DEFS.H
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 */
 Pascal version
}

const
  GEN_STACK=1120;
  MAX_PLY=32;
  HIST_STACK=400;

type
  TBoardSide=(Light,Dark,bsEmpty);
  TSide=Light..Dark;

  TBoardPiece=(Pawn,Knight,Bishop,Rook,Queen,King,bpEmpty);
  TPiece=Pawn..King;

const // /* useful squares */
  A1=56;
  B1=57;
  C1=58;
  D1=59;
  E1=60;
  F1=61;
  G1=62;
  H1=63;
  A8=0;
  B8=1;
  C8=2;
  D8=3;
  E8=4;
  F8=5;
  G8=6;
  H8=7;


{/* This is the basic description of a move. promote is what
   piece to promote the pawn to, if the move is a pawn
   promotion. bits describes the move
*/}

type
  TPawnPromoteMove=(ppmCapture,ppmCastle,ppmEnPassCapture,ppmPawn2sq,ppmPawn,ppmPromote);
  TPPMoves=set of TPawnPromoteMove;
  TCastlePermission=(AllowWhiteKingSide,AllowWhiteQueenSide,AllowBlackKingSide,AllowBlackQueenside);
  TCastlePermissions=set of TCastlePermission;

type
  TMove=record
    from:shortint;
    ToNdx:shortint;
    promote:TPiece;
    bits:TPPMoves;
  end;

{/* an element of the move stack. it's just a move with a
   score, so it can be sorted by the search functions. */}

  PGen=^TGen;
  TGen=record
    m:TMove;
    score:longint;
  end;

{/* an element of the history stack, with the information
   necessary to take a move back. */}

  THist=record
    m:TMove;
    Capture:TBoardPiece;
    castle:TCastlePermissions;
    ep:integer;
    fifty:integer;
  end;


procedure init;
procedure gen;
procedure gen_caps;
function attack(sq:integer; s:TSide):boolean;
procedure takeback;
function makemove(m:TMove):boolean;
function in_check(s:TSide):boolean;

{/* the board representation */}

type
  TColorBoard=array[0..63] of TBoardSide;
  TPieceBoard=array[0..63] of TBoardPiece;

var
  color:TColorBoard; {/* LIGHT, DARK, or EMPTY */}
  piece:TPieceBoard; {/* PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING, or EMPTY */}
  side:TSide;  {/* the side to move */}
  xside:TSide; {/* the side not to move */}
  castle:TCastlePermissions; {/* a bitfield with the castle permissions. if 1 is set,
                      white can still castle kingside. 2 is white queenside.
			              	4 is black kingside. 8 is black queenside. */}

  ep:integer;  {/* the en passant square. if white moves e2e4, the en passant
                    square is set to e3, because that's where a pawn would move
              			in an en passant capture */}
  fifty:integer;  {/* the number of moves since a capture or pawn move, used
                      to handle the fifty-move-draw rule */}
  ply:integer;  {/* the number of half-moves (ply) since the
                    root of the search tree */}
  hply:integer;  {/* h for history; the number of ply since the beginning
                     of the game */}

var
{/* gen_dat is some memory for move lists that are created by the move
   generators. The move list for ply n starts at gen_begin[n] of gen_dat.
   gen_end[n] is right after the move list. */}

  gen_dat:array[0..Gen_Stack-1] of TGen;
  gen_begin,gen_end:array[0..Max_Ply-1] of integer;

{/* the history heuristic array (used for move ordering) */}
  history:array[0..63,0..63] of integer;

{/* we need an array of hist_t's so we can take back the
   moves we make */}
   hist_dat:array[0..Hist_Stack-1] of THist;

implementation


{/* Now we have the mailbox array, so called because it looks like a
   mailbox, at least according to Bob Hyatt. This is useful when we
   need to figure out what pieces can go where. Let's say we have a
   rook on square a4 (32) and we want to know if it can move one
   square to the left. We subtract 1, and we get 31 (h5). The rook
   obviously can't move to h5, but we don't know that without doing
   a lot of annoying work. Sooooo, what we do is figure out a4's
   mailbox number, which is 61. Then we subtract 1 from 61 (60) and
   see what mailbox[60] is. In this case, it's -1, so it's out of
   bounds and we can forget it. You can see how mailbox[] is used
   in attack() in board.c. */}

var
  mailbox:array[0..119] of integer = (
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1,  0,  1,  2,  3,  4,  5,  6,  7, -1,
	 -1,  8,  9, 10, 11, 12, 13, 14, 15, -1,
	 -1, 16, 17, 18, 19, 20, 21, 22, 23, -1,
	 -1, 24, 25, 26, 27, 28, 29, 30, 31, -1,
	 -1, 32, 33, 34, 35, 36, 37, 38, 39, -1,
	 -1, 40, 41, 42, 43, 44, 45, 46, 47, -1,
	 -1, 48, 49, 50, 51, 52, 53, 54, 55, -1,
	 -1, 56, 57, 58, 59, 60, 61, 62, 63, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
);

  mailbox64:array[0..63] of integer = (
	21, 22, 23, 24, 25, 26, 27, 28,
	31, 32, 33, 34, 35, 36, 37, 38,
	41, 42, 43, 44, 45, 46, 47, 48,
	51, 52, 53, 54, 55, 56, 57, 58,
	61, 62, 63, 64, 65, 66, 67, 68,
	71, 72, 73, 74, 75, 76, 77, 78,
	81, 82, 83, 84, 85, 86, 87, 88,
	91, 92, 93, 94, 95, 96, 97, 98
);


{/* slide, offsets, and offset are basically the vectors that
   pieces can move in. If slide for the piece is FALSE, it can
   only move one square in any one direction. offsets is the
   number of directions it can move in, and offset is an array
   of the actual directions. */}

  slide:array[Pawn..King] of boolean = (
	FALSE, FALSE, TRUE, TRUE, TRUE, FALSE
);

  offsets:array[Pawn..King] of integer = (
	0, 8, 4, 4, 8, 8
);

  offset:array[Pawn..King,0..7] of integer = (
	( 0, 0, 0, 0, 0, 0, 0, 0 ),
	( -21, -19, -12, -8, 8, 12, 19, 21 ),
	( -11, -9, 9, 11, 0, 0, 0, 0 ),
	( -10, -1, 1, 10, 0, 0, 0, 0 ),
	( -11, -10, -9, -1, 1, 9, 10, 11 ),
	( -11, -10, -9, -1, 1, 9, 10, 11 )
);


{/* This is the castle_mask array. We can use it to determine
   the castling permissions after a move. What we do is
   logical-AND the castle bits with the castle_mask bits for
   both of the move's squares. Let's say castle is 1, meaning
   that white can still castle kingside. Now we play a move
   where the rook on h1 gets captured. We AND castle with
   castle_mask[63], so we have 1&14, and castle becomes 0 and
   white can't castle kingside anymore. */}

  castle_mask:array[0..63] of byte = (
	 7, 15, 15, 15,  3, 15, 15, 11,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	13, 15, 15, 15, 12, 15, 15, 14
);


{/* the initial board state */}

init_color: TColorBoard = (
  Dark,Dark,Dark,Dark,Dark,Dark,Dark,Dark,
  Dark,Dark,Dark,Dark,Dark,Dark,Dark,Dark,
  bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,
  bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,
  bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,
  bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,bsEmpty,
  Light,Light,Light,Light,Light,Light,Light,Light,
  Light,Light,Light,Light,Light,Light,Light,Light
);

init_piece:TPieceBoard = (
	Rook, Knight, Bishop, Queen, King, Bishop, Knight, Rook,
	Pawn, Pawn, Pawn, Pawn, Pawn, Pawn, Pawn, Pawn,
	bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty,
	bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty,
	bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty,
	bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty, bpEmpty,
	Pawn, Pawn, Pawn, Pawn, Pawn, Pawn, Pawn, Pawn,
	Rook, Knight, Bishop, Queen, King, Bishop, Knight, Rook
);

{/* init() sets the board to the initial game state */}

procedure init;
begin
  color:=init_color;
  piece:=init_piece;
	side := LIGHT;
	xside := DARK;
	castle := [AllowWhiteKingSide..AllowBlackQueenSide];
	ep := -1;
	fifty := 0;
	ply := 0;
	hply := 0;
	gen_begin[0] := 0;
end;

{/* in_check() returns TRUE if side s is in check and FALSE
   otherwise. It just scans the board to find side s's king
   and calls attack() to see if it's being attacked. */}

function OtherSide(s:TSide):TSide;
begin
  result:=TSide(integer(s) xor 1);
end;

function in_check(s:TSide):boolean;
var i:integer;
begin
  i:=0;
  while (i<64) and not ((color[i]=s) and (piece[i] = KING)) do
    inc(i);
  if i<64 then
		result:=attack(i,OtherSide(s))
  else
  	result:=true;  {/* shouldn't get here */}
end;


{/* attack() returns TRUE if square sq is being attacked by side
   s and FALSE otherwise. */}

function IsDiagLight(i,sq:integer):boolean;
begin
  if ((i and 7 <> 0) and (i - 9 = sq)) or ((i and 7 <> 7) and (i - 7 = sq)) then
    result:=true
  else
    result:=false;
end;

function IsDiagdark(i,sq:integer):boolean;
begin
	if ((i and 7 <> 0) and (i + 7 = sq)) or ((i and 7 <> 7) and (i + 9 = sq)) then
    result:=true
  else
    result:=false;
end;

function ScanMailBoxes(i,sq:integer):boolean;
var
  j,n:integer;
  p:TPiece;
begin
  result:=false;
  j:=0;
  p:=piece[i]; // this is an optimization !!!!!!
  while not result and (j<offsets[p]) do
  begin
    n:=i;
    repeat
      n := mailbox[mailbox64[n] + offset[p][j]];
    until (n=-1) or (n=sq) or (color[n] <> bsEmpty) or (not slide[p]);
    if (n = sq) then
      result:=TRUE;
    inc(j);
  end
end;

function attack(sq:integer; s:TSide):boolean;
var
	i:integer;
begin
	result:=false;
  i:=0;
  repeat
		if (color[i] = s) then
			if (piece[i] = PAWN) then
				if (s = LIGHT) then
          result:=IsDiagLight(i,sq)
				else
          result:=IsDiagDark(i,sq)
			else
        result:=scanMailBoxes(i,sq);
    inc(i);
  until result or (i=64);
end;

{/* gen_promote() is just like gen_push(), only it puts 4 moves
   on the move stack, one for each possible promotion piece */}

procedure gen_promote(from, ToNdx:integer; bits:TPPMoves);
var
  i:TPiece;
begin
  for i:=Knight to Queen do
  begin
		with gen_dat[gen_end[ply]] do
    begin
  		m.from := from;
	  	m.ToNdx := ToNdx;
		  m.promote := i;
  		m.bits := bits+[ppmPromote];
	  	score := 1000000 + (ord(i) * 10);
    end;
		inc(gen_end[ply]);
	end
end;


{/* gen_push() puts a move on the move stack, unless it's a
   pawn promotion that needs to be handled by gen_promote().
   It also assigns a score to the move for alpha-beta move
   ordering. If the move is a capture, it uses MVV/LVA
   (Most Valuable Victim/Least Valuable Attacker). Otherwise,
   it uses the move's history heuristic value. Note that
   1,000,000 is added to a capture move's score, so it
   always gets ordered above a "normal" move. */}

procedure gen_push(from, ToNdx:integer; bits:TPPMoves);
begin
	if ppmPawn in bits then
  begin
		if (side = LIGHT) then
    begin
			if (ToNdx <= H8) then
      begin
				gen_promote(from, ToNdx, bits);
				exit;
			end
		end
		else
    begin
			if (ToNdx >= A1) then
      begin
				gen_promote(from, ToNdx, bits);
				exit;
			end
		end
	end;
	with gen_dat[gen_end[ply]] do
  begin
	  m.from := from;
	  m.ToNdx := ToNdx;
	  m.promote := Pawn;
	  m.bits := bits;
  	if (color[ToNdx] <> bsEmpty) then
	  	score := 1000000 + (ord(piece[ToNdx]) * 10) - ord(piece[from])
  	else
	  	score := history[from][ToNdx];
  end;
	inc(gen_end[ply]);
end;


{/* gen() generates pseudo-legal moves for the current position.
   It scans the board to find friendly pieces and then determines
   what squares they attack. When it finds a piece/square
   combination, it calls gen_push to put the move on the "move
   stack." */}

procedure gen;
  var
    i,j,n:integer;
begin
	{/* so far, we have no moves for the current ply */}
	gen_end[ply] := gen_begin[ply];

  for i:=0 to 63 do
		if (color[i] = side) then
    begin
			if (piece[i] = PAWN) then
      begin
				if (side = LIGHT) then
        begin
					if (((i and 7) <> 0) and (color[i - 9] = DARK)) then
						gen_push(i, i - 9, [ppmPawn,ppmCapture]);
					if (((i and 7) <> 7) and (color[i - 7] = DARK)) then
						gen_push(i, i - 7, [ppmPawn,ppmCapture]);
					if (color[i - 8] = bsEmpty) then
          begin
						gen_push(i, i - 8, [ppmPawn]);
						if ((i >= 48) and (color[i - 16] = bsEmpty)) then
							gen_push(i, i - 16, [ppmPawn,ppmPawn2sq]);
					end
				end
				else
        begin
					if (((i and 7) <> 0) and (color[i + 7] = LIGHT)) then
						gen_push(i, i + 7, [ppmPawn,ppmCapture]);
					if (((i and 7) <> 7) and (color[i + 9] = LIGHT)) then
						gen_push(i, i + 9, [ppmPawn,ppmCapture]);
					if (color[i + 8] = bsEmpty) then
          begin
						gen_push(i, i + 8, [ppmPawn]);
						if ((i <= 15) and (color[i + 16] = bsEmpty)) then
							gen_push(i, i + 16, [ppmPawn,ppmPawn2sq]);
					end
				end
			end
			else
        for j:=0 to offsets[piece[i]]-1 do
        begin
          n:=i;
          repeat
						n := mailbox[mailbox64[n] + offset[piece[i],j]];
						if (n = -1) then
							break;
						if (color[n] <> bsEmpty) then
            begin
							if (color[n] = xside) then
								gen_push(i, n, [ppmCapture]);
							break;
						end;
						gen_push(i, n, []);
						if (slide[piece[i]]<>True) then
							break;
          until system.false;
				end;
		end;

	{/* generate castle moves */}
	if (side = LIGHT) then
  begin
		if AllowWhiteKingSide in castle then
			gen_push(E1, G1, [ppmCastle]);
		if AllowWhiteQueenSide in castle then
			gen_push(E1, C1, [ppmCastle]);
	end
	else
  begin
		if AllowBlackKingSide in castle then
			gen_push(E8, G8, [ppmCastle]);
		if AllowBlackQueenSide in castle then
			gen_push(E8, C8, [ppmCastle]);
	end;

	{/* generate en passant moves */}
	if (ep <> -1) then
  begin
		if (side = LIGHT) then
    begin
			if (((ep and 7) <> 0) and (color[ep + 7] = LIGHT) and (piece[ep + 7] = PAWN)) then
				gen_push(ep + 7, ep,[ppmPawn,ppmEnPassCapture,ppmCapture]);
			if (((ep and 7) <> 7) and (color[ep + 9] = LIGHT) and (piece[ep + 9] = PAWN)) then
				gen_push(ep + 9, ep, [ppmPawn,ppmEnPassCapture,ppmCapture]);
		end
		else
    begin
			if (((ep and 7) <> 0) and (color[ep - 9] = DARK) and (piece[ep - 9] = PAWN)) then
				gen_push(ep - 9, ep, [ppmPawn,ppmEnPassCapture,ppmCapture]);
			if (((ep and 7) <> 7) and (color[ep - 7] = DARK) and (piece[ep - 7] = PAWN)) then
				gen_push(ep - 7, ep, [ppmPawn,ppmEnPassCapture,ppmCapture]);
		end
	end;

	{/* the next ply's moves need to start where the current
	   ply's end */}
	gen_begin[ply + 1] := gen_end[ply];
end;


{/* gen_caps() is basically a copy of gen() that's modified to
   only generate capture and promote moves. It's used by the
   quiescence search. */}

procedure gen_caps;
var
  i,j,n:integer;
begin
	gen_end[ply] := gen_begin[ply];
  for i:=0 to 63 do
		if (color[i] = side) then
    begin
			if (piece[i]=PAWN) then
      begin
				if (side = LIGHT) then
        begin
					if (((i and 7) <> 0) and (color[i - 9] = DARK)) then
						gen_push(i, i - 9, [ppmPawn,ppmCapture]);
					if (((i and 7) <> 7) and (color[i - 7] = DARK)) then
						gen_push(i, i - 7, [ppmPawn,ppmCapture]);
					if ((i <= 15) and (color[i - 8] = bsEmpty)) then
						gen_push(i, i - 8, [ppmPawn]);
				end;
				if (side = DARK) then
        begin
					if (((i and 7) <> 0) and (color[i + 7] = LIGHT)) then
						gen_push(i, i + 7, [ppmPawn,ppmCapture]);
					if (((i and 7) <> 7) and (color[i + 9] = LIGHT)) then
						gen_push(i, i + 9, [ppmPawn,ppmCapture]);
					if ((i >= 48) and (color[i + 8] = bsEmpty)) then
						gen_push(i, i + 8, [ppmPawn]);
				end
			end
			else
        for j:=0 to offsets[piece[i]]-1 do
        begin
          n:=i;
          repeat
						n := mailbox[mailbox64[n] + offset[piece[i],j]];
						if (n = -1) then
							break;
						if (color[n] <> bsEmpty) then
            begin
							if (color[n] = xside) then
								gen_push(i, n, [ppmCapture]);
							break;
						end;
						if (slide[piece[i]]<>True) then
							break;
					until system.false;
        end
		end;
	if (ep <> -1) then
  begin
		if (side = LIGHT) then
    begin
			if (((ep and 7) <> 0) and (color[ep + 7] = LIGHT) and (piece[ep + 7] = PAWN)) then
				gen_push(ep + 7, ep, [ppmPawn,ppmEnPassCapture,ppmCapture]);
			if (((ep and 7) <> 7) and (color[ep + 9] = LIGHT) and (piece[ep + 9] = PAWN)) then
				gen_push(ep + 9, ep, [ppmPawn,ppmEnPassCapture,ppmCapture]);
		end
		else
    begin
			if (((ep and 7) <> 0) and (color[ep - 9] = DARK) and (piece[ep - 9] = PAWN)) then
				gen_push(ep - 9, ep, [ppmPawn,ppmEnPassCapture,ppmCapture]);
			if (((ep and 7) <> 7) and (color[ep - 7] = DARK) and (piece[ep - 7] = PAWN)) then
				gen_push(ep - 7, ep, [ppmPawn,ppmEnPassCapture,ppmCapture]);
		end
	end;
	gen_begin[ply + 1] := gen_end[ply];
end;


{/* takeback() is very similar to makemove(), only backwards :)  */}

procedure takeback;
var
  m:TMove;
  from,ToNdx:integer;
begin
	side :=TSide(integer(side) xor 1);
	xside :=TSide(integer(xside) xor 1);
	dec(ply);
	dec(hply);
	m := hist_dat[hply].m;
	castle := hist_dat[hply].castle;
	ep := hist_dat[hply].ep;
	fifty := hist_dat[hply].fifty;
	color[m.from] := side;
	if ppmPromote in m.bits then
		piece[m.from] := PAWN
	else
		piece[m.from] := piece[m.ToNdx];
	if (hist_dat[hply].capture = bpEmpty) then
  begin
		color[m.ToNdx] := bsEmpty;
		piece[m.ToNdx] := bpEmpty;
	end
	else
  begin
		color[m.ToNdx] := xside;
		piece[m.ToNdx] := hist_dat[hply].capture;
	end;
	if ppmCastle in m.bits then
  begin
		case (m.ToNdx) of
			62: begin
				from := F1;
				ToNdx := H1;
				end;
			58: begin
				from := D1;
				ToNdx := A1;
				end;
			6: begin
				from := F8;
				ToNdx := H8;
				end;
			2: begin
				from := D8;
				ToNdx := A8;
				end;
      else
      begin {/* shouldn't get here */}
				from := -1;
				ToNdx := -1;
      end;
		end;
		color[ToNdx] := side;
		piece[ToNdx] := ROOK;
		color[from] := bsEmpty;
		piece[from] := bpEmpty;
	end;
	if ppmEnPassCapture in m.bits then
  begin
		if (side = LIGHT) then
    begin
			color[m.ToNdx + 8] := xside;
			piece[m.ToNdx + 8] := PAWN;
		end
		else
    begin
			color[m.ToNdx - 8] := xside;
			piece[m.ToNdx - 8] := PAWN;
		end
	end
end;


{/* makemove() makes a move. If the move is illegal, it
   undoes whatever it did and returns FALSE. Otherwise, it
   returns TRUE. */}

function makemove(m:TMove):boolean;
var
		from, ToNdx:integer;
begin
	{/* test to see if a castle move is legal and move the rook
	   (the king is moved with the usual move code later) */}
	if ppmCastle in m.bits then
  begin
		if (in_check(side)=True) then
    begin result:=False; exit; end;
    case m.ToNdx of
			62: begin
				if ((color[F1] <> bsEmpty) or (color[G1] <> bsEmpty) or
						(true=attack(F1, xside)) or (true=attack(G1, xside))) then
          begin result:=False; exit; end;
				from := H1;
				ToNdx := F1;
				end;
			58: begin
				if ((color[B1] <> bsEmpty) or (color[C1] <> bsEmpty) or (color[D1] <> bsEmpty) or
						(true=attack(C1, xside)) or (true=attack(D1, xside))) then
          begin result:=False; exit; end;
				from := A1;
				ToNdx := D1;
				end;
			6: begin
				if ((color[F8] <> bsEmpty) or (color[G8] <> bsEmpty) or
						(true=attack(F8, xside)) or (true=attack(G8, xside))) then
          begin result:=False; exit; end;
				from := H8;
				ToNdx := F8;
				end;
			2: begin
				if ((color[B8] <> bsEmpty) or (color[C8] <> bsEmpty) or (color[D8] <> bsEmpty) or
						(true=attack(C8, xside)) or (true=attack(D8, xside))) then
          begin result:=False; exit; end;
				from := A8;
				ToNdx := D8;
				end;
      else
      begin
			  {/* shouldn't get here */}
				from := -1;
				ToNdx := -1;
      end;
		end;
		color[ToNdx] := color[from];
		piece[ToNdx] := piece[from];
		color[from] := bsEmpty;
		piece[from] := bpEmpty;
	end;

	{/* back up information so we can take the move back later. */}
	hist_dat[hply].m := m;
	hist_dat[hply].capture := piece[m.ToNdx];
	hist_dat[hply].castle := castle;
	hist_dat[hply].ep := ep;
	hist_dat[hply].fifty := fifty;
	inc(ply);
	inc(hply);

	{/* update the castle, en passant, and
	   fifty-move-draw variables */}

	castle :=castle - TCastlePermissions(castle_mask[m.from]) - TCastlePermissions(castle_mask[m.ToNdx]);
	if ppmPawn2sq in m.bits then
  begin
		if (side = LIGHT) then
			ep := m.ToNdx + 8
		else
			ep := m.ToNdx - 8;
	end
	else
		ep := -1;
	if (ppmPawn in m.bits) and  (ppmCapture in m.bits) then
		fifty := 0
	else
		inc(fifty);

	{/* move the piece */}
	color[m.ToNdx] := side;
	if ppmPromote in m.bits then
		piece[m.ToNdx] := m.promote
	else
		piece[m.ToNdx] := piece[m.from];
	color[m.from] := bsEmpty;
	piece[m.from] := bpEmpty;

	{/* erase the pawn if this is an en passant move */}
	if ppmEnPassCapture in m.bits then
  begin
		if (side = LIGHT) then
    begin
			color[m.ToNdx + 8] := bsEmpty;
			piece[m.ToNdx + 8] := bpEmpty;
		end
		else
    begin
			color[m.ToNdx - 8] := bsEmpty;
			piece[m.ToNdx - 8] := bpEmpty;
		end
	end;

	{/* switch sides and test for legality (if we can capture
	   the other guy's king, it's an illegal position and
	   we need to take the move back) */}
	side :=TSide(integer(side) xor 1);
	xside :=TSide(integer(xside) xor 1);
	if (in_check(xside)=true) then
  begin
		takeback();
		result:=FALSE;
    exit;
	end;
	result:=TRUE;
end;


end.
