unit evals;

interface

{/*
 *	EVAL.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 */}
//Pascal version

uses board;

function eval:integer;
function eval_light_pawn(sq:integer):integer;
function eval_dark_pawn(sq:integer):integer;
function eval_light_king(sq:integer):integer;
function eval_dark_king(sq:integer):integer;

implementation

const
  DOUBLED_PAWN_PENALTY=10;
  ISOLATED_PAWN_PENALTY=		20;
  BACKWARDS_PAWN_PENALTY=		8;
  PASSED_PAWN_BONUS=			20;
  ROOK_SEMI_OPEN_FILE_BONUS=	10;
  ROOK_OPEN_FILE_BONUS=		5;
  ROOK_ON_SEVENTH_BONUS=		20;

{/* the values of the pieces */}
  piece_value:array[TPiece] of integer = (
	100, 300, 300, 500, 900, 0
);

{/* The "pcsq" arrays are piece/square tables. They're values
   added to the material value of the piece based on the
   location of the piece. */}

  pawn_pcsq:array[0..63] of integer = (
	  0,   0,   0,   0,   0,   0,   0,   0,
	  5,  10,  15,  20,  20,  15,  10,   5,
	  4,   8,  12,  16,  16,  12,   8,   4,
	  3,   6,   9,  12,  12,   9,   6,   3,
	  2,   4,   6,   8,   8,   6,   4,   2,
	  1,   2,   3, -10, -10,   3,   2,   1,
	  0,   0,   0, -40, -40,   0,   0,   0,
	  0,   0,   0,   0,   0,   0,   0,   0
);

  knight_pcsq:array[0..63] of integer = (
	-10, -10, -10, -10, -10, -10, -10, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10, -30, -10, -10, -10, -10, -30, -10
);

  bishop_pcsq:array[0..63] of integer = (
	-10, -10, -10, -10, -10, -10, -10, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10, -10, -20, -10, -10, -20, -10, -10
);

  king_pcsq:array[0..63] of integer = (
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
	  0,   4,   8,  -4,   0,  -4,   8,   4
);

  king_endgame_pcsq:array[0..63] of integer = (
	  0,  10,  20,  30,  30,  20,  10,   0,
	 10,  20,  30,  40,  40,  30,  20,  10,
	 20,  30,  40,  50,  50,  40,  30,  20,
	 30,  40,  50,  60,  60,  50,  40,  30,
	 30,  40,  50,  60,  60,  50,  40,  30,
	 20,  30,  40,  50,  50,  40,  30,  20,
	 10,  20,  30,  40,  40,  30,  20,  10,
	  0,  10,  20,  30,  30,  20,  10,   0
);

{/* The flip array is used to calculate the piece/square
   values for DARK pieces. The piece/square value of a
   LIGHT pawn is pawn_pcsq[sq] and the value of a DARK
   pawn is pawn_pcsq[flip[sq]] */}
  flip:array[0..63] of integer = (
	 56,  57,  58,  59,  60,  61,  62,  63,
	 48,  49,  50,  51,  52,  53,  54,  55,
	 40,  41,  42,  43,  44,  45,  46,  47,
	 32,  33,  34,  35,  36,  37,  38,  39,
	 24,  25,  26,  27,  28,  29,  30,  31,
	 16,  17,  18,  19,  20,  21,  22,  23,
	  8,   9,  10,  11,  12,  13,  14,  15,
	  0,   1,   2,   3,   4,   5,   6,   7
);

var
{/* pawn_count[x][y] is the number of pawns of color x on file y.
   The array has two extra files (0 and 9) that will always be 0
   so we can use the same pawn evaluation code on every pawn without
   worrying about going out of bounds in the array. */}
  pawn_count:array[Light..Dark,0..9] of integer;

{/* pawn_rank[x][y] is the rank of the least advanced pawn of color x
   on file y. If there isn't a pawn on that file, we assume it
   advanced off the board. This is a kludge that makes some of the
   pawn evaluation code work. This array has two extra files for the
   same reason pawn_count does. */}
  pawn_rank:array[Light..Dark,0..9] of integer;

  piece_mat:array[Light..Dark] of integer;  {/* the value of a side's pieces */}
  pawn_mat:array[Light..Dark] of integer;  {/* the value of a side's pawns */}

function eval:integer;
var
	i:integer;
	f:integer;  {/* file */}
	score:array[Light..Dark] of integer;  {/* each side's score */}

begin
	{/* this is the first pass: set up pawn_count, pawn_rank,
	   piece_mat, and pawn_mat. */}
  for i:=0 to 9 do
  begin
		pawn_count[LIGHT][i] := 0;
		pawn_count[DARK][i] := 0;
		pawn_rank[LIGHT][i] := -1;
		pawn_rank[DARK][i] := 8;
	end;
	piece_mat[LIGHT] := 0;
	piece_mat[DARK] := 0;
	pawn_mat[LIGHT] := 0;
	pawn_mat[DARK] := 0;
  for i:=0 to 63 do
  begin
		if (color[i] = bsEmpty) then
			continue;
		if (piece[i] = PAWN) then
    begin
			inc(pawn_mat[color[i]],piece_value[PAWN]);
			f := (i and 7) + 1;  {/* add 1 because of the extra file in the array */}
			if (color[i] = LIGHT) then
      begin
				inc(pawn_count[LIGHT][f]);
				if (pawn_rank[LIGHT][f] < (i shr 3)) then
					pawn_rank[LIGHT][f] := (i shr 3);
			end
			else
      begin
				inc(pawn_count[DARK][f]);
				if (pawn_rank[DARK][f] > (i shr 3)) then
					pawn_rank[DARK][f] := (i shr 3);
			end
		end
		else
			inc(piece_mat[color[i]],piece_value[piece[i]]);
	end;

	{/* this is the second pass: evaluate each piece */}
	score[LIGHT] := piece_mat[LIGHT] + pawn_mat[LIGHT];
	score[DARK] := piece_mat[DARK] + pawn_mat[DARK];
  for i:=0 to 63 do
  begin
		if (color[i] = bsEmpty) then
			continue;
		if (color[i] = LIGHT) then
    begin
			case (piece[i]) of
				PAWN: begin
					inc(score[LIGHT],eval_light_pawn(i));
					end;
				KNIGHT: begin
					inc(score[LIGHT],knight_pcsq[i]);
					end;
				BISHOP: begin
					inc(score[LIGHT],bishop_pcsq[i]);
					end;
				ROOK: begin
					if (pawn_count[LIGHT][(i and 7) + 1]=0) then
          begin
						inc(score[LIGHT],ROOK_SEMI_OPEN_FILE_BONUS);
						if (pawn_count[DARK][(i and 7) + 1]=0) then
							inc(score[LIGHT],ROOK_OPEN_FILE_BONUS);
					end;
					if ((i shr 3) = 1) then
						inc(score[LIGHT],ROOK_ON_SEVENTH_BONUS);
					end;
				KING: begin
					if (piece_mat[DARK] <= 1200) then
						inc(score[LIGHT],king_endgame_pcsq[i])
					else
						inc(score[LIGHT],eval_light_king(i));
					end;
			end
		end
		else begin
			case (piece[i]) of
				PAWN: begin
					inc(score[DARK],eval_dark_pawn(i));
					end;
				KNIGHT: begin
					inc(score[DARK],knight_pcsq[flip[i]]);
					end;
				BISHOP: begin
					inc(score[DARK],bishop_pcsq[flip[i]]);
					end;
				ROOK: begin
					if (pawn_count[DARK][(i and 7) + 1]=0) then
          begin
						inc(score[DARK],ROOK_SEMI_OPEN_FILE_BONUS);
						if (pawn_count[LIGHT][(i and 7) + 1]=0) then
							inc(score[DARK],ROOK_OPEN_FILE_BONUS);
					end;
					if ((i shr 3) = 6) then
						inc(score[DARK],ROOK_ON_SEVENTH_BONUS);
					end;
				KING: begin
					if (piece_mat[LIGHT] <= 1200) then
						inc(score[DARK],king_endgame_pcsq[flip[i]])
					else
						inc(score[DARK],eval_dark_king(i));
					end;
			end
		end
	end;

	{/* the score[] array is set, now return the score relative
	   to the side to move */}
	if (side = LIGHT) then
		result:=score[LIGHT] - score[DARK]
  else
  	result:=score[DARK] - score[LIGHT];
end;

function eval_light_pawn(sq:integer):integer;
var
	r:integer;  {/* the value to return */}
	f:integer;  {/* the pawn's file */}
begin

	r := 0;
	f := (sq and 7) + 1;

	inc(r,pawn_pcsq[sq]);

	{/* if there's a pawn behind this one, it's doubled */}
	if (pawn_rank[LIGHT][f] > (sq shr 3)) then
		dec(r,DOUBLED_PAWN_PENALTY);

	{/* if there aren't any friendly pawns on either side of
	   this one, it's isolated */}
	if ((pawn_count[LIGHT][f - 1]=0) and (pawn_count[LIGHT][f + 1]=0)) then
		dec(r,ISOLATED_PAWN_PENALTY)

	{/* if it's not isolated, it might be backwards */}
	else if ((pawn_rank[LIGHT][f - 1] < (sq shr 3)) and (pawn_rank[LIGHT][f + 1] < (sq shr 3))) then
		dec(r,BACKWARDS_PAWN_PENALTY);

	{/* add a bonus if the pawn is passed */}
	if ((pawn_rank[DARK][f - 1] >= (sq shr 3)) and
			(pawn_rank[DARK][f] >= (sq shr 3)) and
			(pawn_rank[DARK][f + 1] >= (sq shr 3))) then
		inc(r,(7 - (sq shr 3)) * PASSED_PAWN_BONUS);

	result:=r;
end;

function eval_dark_pawn(sq:integer):integer;
var
	r:integer;  {/* the value to return */}
	f:integer;  {/* the pawn's file */}
begin
	r := 0;
	f := (sq and 7) + 1;

	inc(r,pawn_pcsq[flip[sq]]);

	{/* if there's a pawn behind this one, it's doubled */}
	if (pawn_rank[DARK][f] < (sq shr 3)) then
		dec(r,DOUBLED_PAWN_PENALTY);

	{/* if there aren't any friendly pawns on either side of
	   this one, it's isolated */}
	if ((pawn_count[DARK][f - 1]=0) and
			(pawn_count[DARK][f + 1]=0)) then
		dec(r,ISOLATED_PAWN_PENALTY)

	{/* if it's not isolated, it might be backwards */}
	else if ((pawn_rank[DARK][f - 1] > (sq shr 3)) and
			(pawn_rank[DARK][f + 1] > (sq shr 3))) then
		dec(r,BACKWARDS_PAWN_PENALTY);

	{/* add a bonus if the pawn is passed */}
	if ((pawn_rank[LIGHT][f - 1] <= (sq shr 3)) and
			(pawn_rank[LIGHT][f] <= (sq shr 3)) and
			(pawn_rank[LIGHT][f + 1] <= (sq shr 3))) then
		inc(r,(sq shr 3) * PASSED_PAWN_BONUS);

	result:=r;
end;


{/* The following two functions are the king safety functions.
   Here's the idea:
   First, the piece/square value is added to r (r is the value
     that's returned)
   Second, the pawns in front of the king are evaluated. If the
     king is on the queenside, the pawns on files a, b, and c
	 are evaluated. If the king is on the kingside, the pawns
	 on files f, g, and h are evaluated. The evaluation of each
	 file works like this: if there isn't a friendly pawn on the
	 file, it's considered open and a penalty is assessed. If
	 there is a friendly pawn on the file, and it hasn't been
	 pushed, nothing happens. If it's been pushed one square,
	 a penalty is assessed. If it's been pushed more than one
	 square, a penalty (usually bigger) is assessed. After this
	 is done for the three files, the enemy pawns are examined.
	 If they're getting too close to the king, it's a "pawn
	 storm" and penalties are assessed. If the king isn't on
	 the kingside or the queenside, i.e., if it's on one of the
	 two center files, then the three files closest to it are
	 examined. A penalty is assessed for each one of these files
	 that's empty.
   Finally, the score is multiplied by a scalar of the opponent's
     material. The rationale for this is that if there are problems
	 with your king safety and your opponent has a lot of pieces,
	 you're in trouble because it's easy for your opponent to
	 attack you. On the other hand, if you have problems with your
	 king safety and your opponent has no pieces, it doesn't really
	 matter, because he can't attack you. */}

function eval_light_king(sq:integer):integer;
var
	r:integer;  {/* the value to return */}
	i:integer;
begin
	r := king_pcsq[sq];

	if ((sq and 7) < 3) then
  begin

		{/* a file */}
		if (pawn_count[LIGHT][1]<>0) then
    begin
			if (pawn_rank[LIGHT][1] = 6)then begin end
			else if (pawn_rank[LIGHT][1] = 5) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[DARK][1]=0) then
				dec(r , 8);

		{/* b file */}
		if (pawn_count[LIGHT][2]<>0) then
    begin
			if (pawn_rank[LIGHT][2] = 6)then begin end
			else if (pawn_rank[LIGHT][2] = 5) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[DARK][2]=0) then
				dec(r , 8);

		{/* c file */}
		if (pawn_count[LIGHT][3]<>0) then
    begin
			if (pawn_rank[LIGHT][3] = 6) then begin end
			else if (pawn_rank[LIGHT][3] =5 ) then
				dec(r , 1)
			else
				dec(r , 3)
		end
		else if (pawn_count[DARK][3]=0) then
				dec(r , 5);

		{/* pawn storm */}
    for i:=1 to 3 do
    begin
			if (pawn_count[DARK][i]=0) then
				continue;
			if (pawn_rank[DARK][i] > 2) then
      begin
				if (pawn_rank[DARK][i] > 3) then
  				dec(r , 2)
				else
	  			dec(r , 1)
			end
		end
	end
	else if ((sq and 7) > 4) then
  begin

		{/* h file */}
		if (pawn_count[LIGHT][8]<>0) then
    begin
			if (pawn_rank[LIGHT][8] = 6) then begin end
			else if (pawn_rank[LIGHT][8] = 5) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[DARK][8]=0) then
				dec(r , 8);

		{/* g file */}
		if (pawn_count[LIGHT][7]<>0) then begin
			if (pawn_rank[LIGHT][7] = 6) then begin end
			else if (pawn_rank[LIGHT][7] = 5) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[DARK][7]=0) then
				dec(r , 8);

		{/* f file */}
		if (pawn_count[LIGHT][6]<>0) then begin
			if (pawn_rank[LIGHT][6] = 6) then begin end
			else if (pawn_rank[LIGHT][6] = 5) then
				dec(r , 1)
			else
				dec(r , 3)
		end
		else if (pawn_count[DARK][6]=0) then
				dec(r , 5);

		{/* pawn storm */}
    for i:=6 to 8 do
    begin
			if (pawn_count[DARK][i]=0) then
				continue;
			if (pawn_rank[DARK][i] > 2) then begin
				if (pawn_rank[DARK][i] > 3) then
				dec(r , 2)
				else
				dec(r , 1)
			end
		end
	end
	else begin
    for i:=(sq and 7) to (sq and 7) +2 do
			if ((pawn_count[LIGHT][i]=0) and
					(pawn_count[DARK][i]=0)) then
				dec(r , 2)
	end;
	r := r * (piece_mat[DARK] div 600);
	result:=r;
end;

function eval_dark_king(sq:integer):integer;
var
	r:integer;  {/* the value to return */}
	i:integer;
begin
	r := king_pcsq[flip[sq]];

	if ((sq and 7) < 3) then
  begin

		{/* a file */}
		if (pawn_count[DARK][1]<>0) then
    begin
			if (pawn_rank[DARK][1] = 1) then begin end
			else if (pawn_rank[DARK][1] = 2) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[LIGHT][1]=0) then
				dec(r , 8);

		{/* b file */}
		if (pawn_count[DARK][2]<>0) then
    begin
			if (pawn_rank[DARK][2] = 1)then begin end
			else if (pawn_rank[DARK][2] = 2) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[LIGHT][2]=0) then
				dec(r , 8);

		{/* c file */}
		if (pawn_count[DARK][3]<>0) then
    begin
			if (pawn_rank[DARK][3] = 1)then begin end
			else if (pawn_rank[DARK][3] = 2) then
				dec(r , 1)
			else
				dec(r , 3)
		end
		else if (pawn_count[LIGHT][3]=0) then
				dec(r , 5);

		{/* pawn storm */}
    for i:=1 to 3 do
    begin
			if (pawn_count[LIGHT][i]=0) then
				continue;
			if (pawn_rank[LIGHT][i] < 5) then
      begin
				if (pawn_rank[LIGHT][i] < 4) then
				dec(r , 2)
				else
				dec(r , 1)
			end
		end
	end
	else if ((sq and 7) > 4) then
  begin

		{/* h file */}
		if (pawn_count[DARK][8]<>0) then
    begin
			if (pawn_rank[DARK][8] = 1)then begin end
			else if (pawn_rank[DARK][8] = 2) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[LIGHT][8]=0) then
				dec(r , 8);

		{/* g file */}
		if (pawn_count[DARK][7]<>0) then
    begin
			if (pawn_rank[DARK][7] = 1)then begin end
			else if (pawn_rank[DARK][7] = 2) then
				dec(r , 2)
			else
				dec(r , 4)
		end
		else if (pawn_count[LIGHT][7]=0) then
				dec(r , 8);

		{/* f file */}
		if (pawn_count[DARK][6]<>0) then
    begin
			if (pawn_rank[DARK][6] = 1)then begin end
			else if (pawn_rank[DARK][6] = 2) then
				dec(r , 1)
			else
				dec(r , 3)
		end
		else if (pawn_count[LIGHT][6]=0) then
				dec(r , 5);

		{/* pawn storm */}
    for i:=6 to 8 do
    begin
			if (pawn_count[LIGHT][i]=0) then
				continue;
			if (pawn_rank[LIGHT][i] < 5) then
      begin
				if (pawn_rank[LIGHT][i] < 4) then
				dec(r , 2)
				else
				dec(r , 1)
			end
		end
	end
	else
  begin
    for i:= (sq and 7) to (sq and 7)+2 do
			if ((pawn_count[LIGHT][i]=0) and
					(pawn_count[DARK][i]=0)) then
				dec(r , 2)
	end;
	r :=r* (piece_mat[LIGHT] div 600);
	result:=r;
end;

end.
