unit main;

interface

{/*
 *	MAIN.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 */}
// pascal version

//#include <signal.h>
uses defs,data;
//#include <sys/timeb.h>

implementation

{/* get_ms() returns the milliseconds elapsed since midnight,
   January 1, 1970. */}

function get_ms:longint;
var
	timebuffer:timeb;
begin
	ftime(&timebuffer);
	result::=(timebuffer.time * 1000) + timebuffer.millitm;
end;

{/* main() is basically an infinite loop that either calls
   think() when it's the computer's turn to move or prompts
   the user for a command (and deciphers it). */}

procedure main;
var
	computer_side:integer;
	s:array[0..255] of char;
	from, ToNdx, i:integer;
	found:Bool;
begin
	writeln;
	writeln('Tom Kerrigan's Simple Chess Program (TSCP)');
	writeln('version 1.6, 7/17/00');
	writeln('Copyright 1997 Tom Kerrigan');
	writeln;
	writeln('"help" displays a list of commands.');
	writeln;
	init;
	gen;
	computer_side := EMPTY;
	max_time := 1000000;
	max_depth := 4;
  while true do
	begin
		if (side = computer_side) then
    begin  {/* computer's turn */}

			{/* think about the move and make it */}
			think(1);
			if (pv[0][0].u=0) then
      begin
				writeln('(no legal moves)');
				computer_side := EMPTY;
				continue;
			end;
			writeln('Computer's move: %s', move_str(pv[0][0].b));
			makemove(pv[0][0].b);
			ply := 0;
			gen;
			print_result;
			continue;
		end;

		{/* get user input */}
		write('tscp> ');
		scanf('%s', s);
		if (strcmp(s, 'on')=0) then
    begin
			computer_side := side;
			continue;
		end;
		if (strcmp(s, 'off')=0)then
     begin
			computer_side := EMPTY;
			continue;
		end;
		if (strcmp(s, 'st')=0) then
    begin
			scanf('%d', &max_time);
			max_time *:= 1000;
			max_depth := 10000;
			continue;
		end;
		if (strcmp(s, 'sd')=0) then
    begin
			scanf('%d', &max_depth);
			max_time := 1000000;
			continue;
		end;
		if (strcmp(s, 'undo')=0) then
    begin
			if (hply=0) then
				continue;
			computer_side := EMPTY;
			takeback;
			ply := 0;
			gen;
			continue;
		end;
		if (strcmp(s, 'new')=0) then
    begin
			computer_side := EMPTY;
			init;
			gen;
			continue;
		end;
		if (strcmp(s, 'd')=0) then
    begin
			print_board;
			continue;
		end;
		if (strcmp(s, 'bench')=0) then
    begin
			computer_side := EMPTY;
			bench;
			continue;
		end;
		if (strcmp(s, 'bye')=0) then
    begin
			writeln('Share and enjoy!');
			halt(0);
		end
		if (strcmp(s, 'xboard')=0) then
    begin
			xboard;
			halt(0);
		end;
		if (strcmp(s, 'help')=0) then
    begin
			writeln('on - computer plays for the side to move');
			writeln('off - computer stops playing');
			writeln('st n - search for n seconds per move');
			writeln('sd n - search n ply per move');
			writeln('undo - takes back a move');
			writeln('new - starts a new game');
			writeln('d - display the board');
			writeln('bench - run the built-in benchmark');
			writeln('bye - exit the program');
			writeln('xboard - switch to XBoard mode');
			writeln('Enter moves in coordinate notation, e.g., e2e4, e7e8Q');
			continue;
		end;

		{/* maybe the user entered a move? */}
		from := ord(s[0] - 'a');
		inc(from,8 * (8 - ord(s[1] - '0')));
		ToNdx := ord(s[2] - 'a');
		inc(ToNdx,8 * (8 - ord(s[3] - '0')));

		{/* loop through the moves to see if it's legal */}
		found := FALSE;
		for i := 0 to gen_end[0]-1 do
			if ((gen_dat[i].m.b.from = from) and (gen_dat[i].m.b.ToNdx = ToNdx)) then
      begin
				found := TRUE;

				{/* get the promotion piece right */}
				if (gen_dat[i].m.b.bits and 32) then
					case (s[4]) of
						'N': break;
						'B': inc(i);
						'R': inc(i,2);
						else:
							inc(i,3)
					end;
				break;
			end;
		if ((found=False) or (makemove(gen_dat[i].m.b)=False)) then
			writeln('Illegal move.');
		else
    begin
			ply := 0;
			gen;
			print_result;
		end
	end
end;


{/* move_str returns a string with move m in coordinate notation */}

function move_str(m:TMoveBytes):pchar;
const
	str:array[0..5] of char='#0#0#0#0#0#0';
var
	c:char;
begin
	if (m.bits and 32) then
  begin
		case (m.promote) of
			KNIGHT:
				c := 'n';
			BISHOP:
				c := 'b';
			ROOK:
				c := 'r';
			else
				c := 'q';
		end
		swrite(str, '%c%d%c%d%c',
				GetFile(m.from) + 'a',
				8 - GetRank(m.from),
				GetFile(m.toNdx) + 'a',
				8 - GetRank(m.toNdx),
				c);
	end
	else
		swrite(str, '%c%d%c%d',
				GetFile(m.from) + 'a',
				8 - GetRank(m.from),
				GetFile(m.toNdx) + 'a',
				8 - GetRank(m.toNdx));
	result:=@str;
end;


{/* print_board() prints the board */}

procedure print_board;
var
	i:integer;
begin
	writeln;
	writeln;
	writeln;
	writeln;
	writeln;
	writeln;
	writeln;
	writeln;
	for i := 0 to 63 do
  begin
		case (color[i]) of
			EMPTY:
				write(' .');
			LIGHT:
				write(' %c', piece_char[piece[i]]);
			DARK:
				write(' %c', piece_char[piece[i]] + ('a' - 'A'));
		end;
		if (((i + 1) % 8 = 0) and (i <> 63)) then
    begin
      writeln;
			write('%d ', 7 - GetRank(i));
    end;
	end
  writeln;
  writeln;
	write('   a b c d e f g h');
  writeln;
  writeln;
end;

{/* print_result() checks to see if the game is over, and if so,
   prints the result. */}

procedure print_result;
var
	i:integer;
begin

	{/* is there a legal move? */}
	for i := 0 to gen_end[0]-1 do
		if (makemove(gen_dat[i].m.b)=true) then
    begin
			takeback;
			break;
		end;
	if (i = gen_end[0]) then
  begin
		if (in_check(side)=True) then
    begin
			if (side = LIGHT) then
				writeln('0-1 {Black mates}');
			else
				writeln('1-0 {White mates}');
		end
		else
			writeln('1/2-1/2 {Stalemate}');
	end
	else if (reps() = 3) then
		write('1/2-1/2 {Draw by repetition}');
	else if (fifty >= 100) then
		write('1/2-1/2 {Draw by fifty move rule}');
end;


{/* bench: This is a little benchmark code that calculates how many
   nodes per second TSCP searches.
   It sets the position to move 17 of Bobby Fischer vs. J. Sherwin,
   New Jersey State Open Championship, 9/2/1957.
   Then it searches five ply three times. It calculates nodes per
   second from the best time. */}

const
  bench_color:array[0..63] of integer = (
	6, 1, 1, 6, 6, 1, 1, 6,
	1, 6, 6, 6, 6, 1, 1, 1,
	6, 1, 6, 1, 1, 6, 1, 6,
	6, 6, 6, 1, 6, 6, 0, 6,
	6, 6, 1, 0, 6, 6, 6, 6,
	6, 6, 0, 6, 6, 6, 0, 6,
	0, 0, 0, 6, 6, 0, 0, 0,
	0, 6, 0, 6, 0, 6, 0, 6
);

  bench_piece:array[0..63] of integer = (
	6, 3, 2, 6, 6, 3, 5, 6,
	0, 6, 6, 6, 6, 0, 0, 0,
	6, 0, 6, 4, 0, 6, 1, 6,
	6, 6, 6, 1, 6, 6, 1, 6,
	6, 6, 0, 0, 6, 6, 6, 6,
	6, 6, 0, 6, 6, 6, 0, 6,
	0, 0, 4, 6, 6, 0, 2, 0,
	3, 6, 2, 6, 3, 6, 5, 6
);

procedure bench;
var
	i:integer;
	t[3]:longint;
	nps:double;

begin
	for i := 0 to 63 do
  begin
		color[i] := bench_color[i];
		piece[i] := bench_piece[i];
	end;
	side := LIGHT;
	xside := DARK;
	castle := 0;
	ep := -1;
	fifty := 0;
	ply := 0;
	hply := 0;
	max_time := 10000;
	max_depth := 5;
	for i := 0 to 2 do
  begin
		think(1);
		t[i] := get_ms() - start_time;
		writeln('Time: %ld ms', t[i]);
	end;
	if (t[1] < t[0]) then
		t[0] := t[1];
	if (t[2] < t[0]) then
		t[0] := t[2];
	writeln;
	writeln('Nodes: %ld', nodes);
	writeln('Best time: %ld ms', t[0]);
	if (t[0] = 0) then
  begin
		writeln('(invalid)');
		exit;
	end;
	nps := nodes;
	nps := nps/t[0];
	nps := nps*1000.0;
	writeln('Nodes per second: %d', round(nps));
end;

end.
