unit mains;

interface

{/*
 *	MAIN.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 */}
// pascal version

uses Windows,board,evals,searches;

procedure main;

implementation


var
{/* the piece letters, for print_board() */}
  piece_char:array[TPiece] of char = (
	'P', 'N', 'B', 'R', 'Q', 'K'
);

{/* print_result() checks to see if the game is over, and if so,
   prints the result. */}

procedure print_result;
var
	i:integer;
begin

	{/* is there a legal move? */}
	for i := 0 to gen_end[0]-1 do
		if makemove(gen_dat[i].m) then
    begin
			takeback;
			break;
		end;
	if (i = gen_end[0]) then
  begin
		if (in_check(side)=True) then
    begin
			if (side = LIGHT) then
				writeln('0-1 {Black mates}')
			else
				writeln('1-0 {White mates}');
		end
		else
			writeln('1/2-1/2 {Stalemate}');
	end
	else if (reps = 3) then
		write('1/2-1/2 {Draw by repetition}')
	else if (fifty >= 100) then
		write('1/2-1/2 {Draw by fifty move rule}');
end;


{/* print_board() prints the board */}

procedure print_board;
var
	i:integer;
begin
	writeln;
	write('8 ');
	for i := 0 to 63 do
  begin
		case (color[i]) of
			bsEmpty:
				write(' .');
			LIGHT:
				write(' ', piece_char[piece[i]]);
			DARK:
				write(' ', chr(ord(piece_char[piece[i]]) + ord('a') - ord('A')));
		end;
		if (((i + 1) mod 8 = 0) and (i <> 63)) then
    begin
      writeln;
			write(7 - (i shr 3),' ');
    end;
	end;
  writeln;
  writeln;
	write('   a b c d e f g h');
  writeln;
  writeln;
end;

{/* bench: This is a little benchmark code that calculates how many
   nodes per second TSCP searches.
   It sets the position to move 17 of Bobby Fischer vs. J. Sherwin,
   New Jersey State Open Championship, 9/2/1957.
   Then it searches five ply three times. It calculates nodes per
   second from the best time. */}

const
  bench_color:array[0..63] of TBoardSide = (
	bsEmpty, Dark, Dark, bsEmpty, bsEmpty, Dark, Dark, bsEmpty,
	Dark, bsEmpty, bsEmpty, bsEmpty, bsEmpty, Dark, Dark, Dark,
	bsEmpty, Dark, bsEmpty, Dark, Dark, bsEmpty, Dark, bsEmpty,
	bsEmpty, bsEmpty, bsEmpty, Dark, bsEmpty, bsEmpty, Light, bsEmpty,
	bsEmpty, bsEmpty, Dark, Light, bsEmpty, bsEmpty, bsEmpty, bsEmpty,
	bsEmpty, bsEmpty, Light, bsEmpty, bsEmpty, bsEmpty, Light, bsEmpty,
	Light, Light, Light, bsEmpty, bsEmpty, Light, Light, Light,
	Light, bsEmpty, Light, bsEmpty, Light, bsEmpty, Light, bsEmpty
);

  bench_piece:array[0..63] of TBoardPiece = (
	bpEmpty, Rook, Bishop, bpEmpty, bpEmpty, Rook, King, bpEmpty,
	Pawn, bpEmpty, bpEmpty, bpEmpty, bpEmpty, Pawn, Pawn, Pawn,
	bpEmpty, Pawn, bpEmpty, Queen, Pawn, bpEmpty, Knight, bpEmpty,
	bpEmpty, bpEmpty, bpEmpty, Knight, bpEmpty, bpEmpty, Knight, bpEmpty,
	bpEmpty, bpEmpty, Pawn, Pawn, bpEmpty, bpEmpty, bpEmpty, bpEmpty,
	bpEmpty, bpEmpty, Pawn, bpEmpty, bpEmpty, bpEmpty, Pawn, bpEmpty,
	Pawn, Pawn, Queen, bpEmpty, bpEmpty, Pawn, Bishop, Pawn,
	Rook, bpEmpty, Bishop, bpEmpty, Rook, bpEmpty, King, bpEmpty
);

procedure bench;
var
	i:integer;
	t:array[0..2] of longint;
	nps:double;

begin
	for i := 0 to 63 do
  begin
		color[i] := bench_color[i];
		piece[i] := bench_piece[i];
	end;
	side := LIGHT;
	xside := DARK;
	castle := [];
	ep := -1;
	fifty := 0;
	ply := 0;
	hply := 0;
	max_time := 10000;
	max_depth := 5;
	for i := 0 to 2 do
  begin
    Stop_search:=false;
		think(1);
		t[i] := GetTickCount - Dword(start_time);
		writeln('Time: ',t[i],' ms');
	end;
	if (t[1] < t[0]) then
		t[0] := t[1];
	if (t[2] < t[0]) then
		t[0] := t[2];
	writeln;
	writeln('Nodes: ',nodes);
	writeln('Best time: ', t[0],' ms');
	if (t[0] = 0) then
  begin
		writeln('(invalid)');
		exit;
	end;
	nps := nodes;
	nps := nps/t[0];
	nps := nps*1000.0;
	writeln('Nodes per second: ', round(nps));
end;

{/* main() is basically an infinite loop that either calls
   think() when it's the computer's turn to move or prompts
   the user for a command (and deciphers it). */}

procedure main;
var
	computer_side:TBoardSide;
	s:string;
	from, ToNdx, i:integer;
	found:Bool;
begin
	writeln;
	writeln('Tom Kerrigan''s Simple Chess Program (TSCP)');
	writeln('version 1.6, 7/17/00');
	writeln('Copyright 1997 Tom Kerrigan');
	writeln;
	writeln('"help" displays a list of commands.');
	writeln;
	init;
	gen;
	computer_side := bsEmpty;
	max_time := 1000000;
	max_depth := 4;
  while system.true do
	begin
		if (side = computer_side) then
    begin  {/* computer's turn */}

			{/* think about the move and make it */}
			think(1);
			if longint(pv[0][0])=0 then
      begin
				writeln('(no legal moves)');
				computer_side := bsEmpty;
				continue;
			end;
			writeln('Computer''s move: ', move_str(pv[0][0]));
			makemove(pv[0][0]);
			ply := 0;
			gen;
			print_result;
			continue;
		end;

		{/* get user input */}
		write('tscp> ');
		readln(s);
		if (s='on') then
    begin
			computer_side := side;
			continue;
		end;
		if (s='off')then
     begin
			computer_side := bsEmpty;
			continue;
		end;
		if (s='st') then
    begin
			readln(max_time);
			max_time :=max_time* 1000;
			max_depth := 10000;
			continue;
		end;
		if (s='sd') then
    begin
			readln(max_depth);
			max_time := 1000000;
			continue;
		end;
		if (s='undo') then
    begin
			if (hply=0) then
				continue;
			computer_side := bsEmpty;
			takeback;
			ply := 0;
			gen;
			continue;
		end;
		if (s='new') then
    begin
			computer_side := bsEmpty;
			init;
			gen;
			continue;
		end;
		if (s='d') then
    begin
			print_board;
			continue;
		end;
		if (s='bench') then
    begin
			computer_side := bsEmpty;
			bench;
			continue;
		end;
		if (s='bye') then
    begin
			writeln('Share and enjoy!');
			halt(0);
		end;
{		if (s, 'xboard')=0) then
    begin
			xboard;
			halt(0);
		end;}
		if (s='help') then
    begin
			writeln('on - computer plays for the side to move');
			writeln('off - computer stops playing');
			writeln('st n - search for n seconds per move');
			writeln('sd n - search n ply per move');
			writeln('undo - takes back a move');
			writeln('new - starts a new game');
			writeln('d - display the board');
			writeln('bench - run the built-in benchmark');
			writeln('bye - exit the program');
			writeln('xboard - switch to XBoard mode');
			writeln('Enter moves in coordinate notation, e.g., e2e4, e7e8Q');
			continue;
		end;

		{/* maybe the user entered a move? */}
		from := ord(s[1]) - ord('a');
		inc(from,8 * (8 - ord(s[2]) - ord('0')));
		ToNdx := ord(s[3]) - ord('a');
		inc(ToNdx,8 * (8 - ord(s[4]) - ord('0')));

		{/* loop through the moves to see if it's legal */}
		found := FALSE;
    i:=0;
    while i<gen_end[0] do
    begin
			if ((gen_dat[i].m.from = from) and (gen_dat[i].m.ToNdx = ToNdx)) then
      begin
				found := TRUE;

				{/* get the promotion piece right */}
				if ppmPromote in gen_dat[i].m.bits then
					case (s[5]) of
						'N': break;
						'B': inc(i);
						'R': inc(i,2);
						else
							inc(i,3)
					end;
				break;
			end;
      inc(i);
    end;
		if ((found=False) or (makemove(gen_dat[i].m)=False)) then
			writeln('Illegal move.')
		else
    begin
			ply := 0;
			gen;
			print_result;
		end
	end
end;


end.
